##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

HOTFIX_ID = 'Hotfix_2002-03-01'

apply_hotfix = 0
try:
    from App.Hotfixes import beforeApplyHotfix
    # Require Zope 2.2.x.
    apply_hotfix = beforeApplyHotfix(HOTFIX_ID, 2, 2, 0)
except ImportError:
    # No hotfix module exists yet.
    apply_hotfix = 1
    from zLOG import LOG, INFO
    LOG('Hotfix', INFO, 'Applying %s.' % HOTFIX_ID)


if apply_hotfix:
    from AccessControl.Owned import UnownableOwner
    from AccessControl import getSecurityManager
    from AccessControl import SpecialUsers
    from Acquisition import aq_get

    def getOwner(self, info=0,
                 aq_get=aq_get, None=None,
                 UnownableOwner=UnownableOwner,
                 getSecurityManager=getSecurityManager,
                 ):
        """Get the owner

        If a true argument is provided, then only the owner path and id are
        returned. Otherwise, the owner object is returned.
        """
        owner=aq_get(self, '_owner', None, 1)
        if info or (owner is None): return owner

        if owner is UnownableOwner: return None

        udb, oid = owner

        # Note that some caching code has been removed to let the
        # the hotfix work for all Zope versions > 2.2.

        root=self.getPhysicalRoot()
        udb=root.unrestrictedTraverse(udb, None)
        if udb is None:
            user = SpecialUsers.nobody
        else:
            user = udb.getUserById(oid, None)
            if user is None: user = SpecialUsers.nobody

        # If we return a user, always return it wrapped, as it might
        # be used in security checks by the security policy!!
        if udb and (user is not SpecialUsers.nobody):
            if not hasattr(user, 'aq_parent'):
                user = user.__of__(udb)
                
        return user

    import AccessControl
    AccessControl.Owned.Owned.getOwner = getOwner
